@@ -8,7 +8,6 @@ from functools import partial, lru_cache
 
 from bids_validator import BIDSValidator
 
-import bids.config
 from ..utils import listify, make_bidsfile
 from ..exceptions import BIDSConflictingValuesError
 
@@ -31,12 +30,12 @@ def _check_path_matches_patterns(path, patterns):
     """Check if the path matches at least one of the provided patterns. """
     if not patterns:
         return False
-    path = os.path.abspath(path)
+    path = path.absolute()
     for patt in patterns:
-        if isinstance(patt, str):
+        if isinstance(patt, Path):
             if path == patt:
                 return True
-        elif patt.search(path):
+        elif patt.search(str(path)):
             return True
     return False
 
@@ -103,11 +102,11 @@ class BIDSLayoutIndexer:
         self._config = list(layout.config.values())
 
         ignore, force = validate_indexing_args(self.ignore, self.force_index,
-                                               self._layout.root)
+                                               self._layout._root)
         self._include_patterns = force
         self._exclude_patterns = ignore
 
-        self._index_dir(self._layout.root, self._config)
+        self._index_dir(self._layout._root, self._config)
         if self.index_metadata:
             self._index_metadata()
 
@@ -141,26 +140,27 @@ class BIDSLayoutIndexer:
 
         # BIDS validator expects absolute paths, but really these are relative
         # to the BIDS project root.
-        to_check = os.path.relpath(f, self._layout.root)
-        to_check = os.path.join(os.path.sep, to_check)
-        to_check = Path(to_check).as_posix()  # bids-validator works with posix paths only
+        to_check = f.relative_to(self._layout._root)
+        # Pretend the path is an absolute path
+        to_check = Path('/') / to_check
+        # bids-validator works with posix paths only
+        to_check = to_check.as_posix()
         return self.validator.is_bids(to_check)
 
     def _index_dir(self, path, config, default_action=None):
 
-        abs_path = os.path.join(self._layout.root, path)
+        abs_path = self._layout._root / path
 
         # Derivative directories must always be added separately
-        # and passed as their own root, so terminate if passed.
-        if abs_path.startswith(os.path.join(self._layout.root, 'derivatives')):
+        if self._layout._root.joinpath('derivatives') in abs_path.parents:
             return
 
         config = list(config)  # Shallow copy
 
         # Check for additional config file in directory
         layout_file = self.config_filename
-        config_file = os.path.join(abs_path, layout_file)
-        if os.path.exists(config_file):
+        config_file = abs_path / layout_file
+        if config_file.exists():
             cfg = Config.load(config_file, session=self.session)
             config.append(cfg)
 
@@ -169,35 +169,33 @@ class BIDSLayoutIndexer:
         for c in config:
             config_entities.update(c.entities)
 
-        for (dirpath, dirnames, filenames) in os.walk(path):
+        # Get a list of first-level subdirectories and files in the path directory
+        _, dirnames, filenames = next(os.walk(path))
 
-            # Set the default inclusion/exclusion directive
-            default = self._validate_dir(dirpath, default=default_action)
+        # Set the default inclusion/exclusion directive
+        default = self._validate_dir(path, default=default_action)
 
-            # If layout configuration file exists, delete it
-            if self.config_filename in filenames:
-                filenames.remove(self.config_filename)
+        # If layout configuration file exists, delete it
+        if self.config_filename in filenames:
+            filenames.remove(self.config_filename)
 
-            for f in filenames:
+        for f in filenames:
+            bf = self._index_file(f, path, config_entities,
+                                  default_action=default)
+            if bf is None:
+                continue
 
-                bf = self._index_file(f, dirpath, config_entities,
-                                      default_action=default)
-                if bf is None:
-                    continue
+        self.session.commit()
 
-            self.session.commit()
+        # Recursively index subdirectories
+        for d in dirnames:
+            d = path / d
+            self._index_dir(d, list(config), default_action=default)
 
-            # Recursively index subdirectories
-            for d in dirnames:
-                d = os.path.join(dirpath, d)
-                self._index_dir(d, list(config), default_action=default)
-
-            # Prevent subdirectory traversal
-            break
 
     def _index_file(self, f, dirpath, entities, default_action=None):
         """Create DB record for file and its tags. """
-        abs_fn = os.path.join(dirpath, f)
+        abs_fn = dirpath / f
 
         # Skip files that fail validation, unless forcibly indexing
         if not self._validate_file(abs_fn, default=default_action):
@@ -226,8 +224,6 @@ class BIDSLayoutIndexer:
     def _index_metadata(self):
         """Index metadata for all files in the BIDS dataset.
         """
-        dot = '.' if bids.config.get_option('extension_initial_dot') else ''
-
         filters = self.filters
 
         if filters:
@@ -238,10 +234,8 @@ class BIDSLayoutIndexer:
             if filters.get(ext_key):
                 filters[ext_key] = listify(filters[ext_key])
                 # ensure json files are being indexed
-                # XXX 0.14: dot always == '.'
-                json_ext = dot + 'json'
-                if json_ext not in filters[ext_key]:
-                    filters[ext_key].append(json_ext)
+                if '.json' not in filters[ext_key]:
+                    filters[ext_key].append('.json')
 
         # Process JSON files first if we're indexing metadata
         all_files = self._layout.get(absolute_paths=True, **filters)
@@ -273,7 +267,7 @@ class BIDSLayoutIndexer:
             with open(path, 'r', encoding='utf-8') as handle:
                 try:
                     return json.load(handle)
-                except json.JSONDecodeError as e:
+                except (UnicodeDecodeError, json.JSONDecodeError) as e:
                     msg = f"Error occurred while trying to decode JSON from file {path}"
                     raise IOError(msg) from e
 
@@ -288,11 +282,11 @@ class BIDSLayoutIndexer:
                     file_data[key] = defaultdict(list)
 
                 payload = None
-                if ext == dot + 'json':
+                if ext == '.json':
                     payload = partial(load_json, bf.path)
 
                 to_store = (file_ents, payload, bf.path)
-                file_data[key][bf.dirname].append(to_store)
+                file_data[key][bf._dirname].append(to_store)
 
         # To avoid integrity errors, track primary keys we've seen
         seen_assocs = set()
@@ -329,8 +323,8 @@ class BIDSLayoutIndexer:
             # add the payload to the stack. Finally, we invert the
             # stack and merge the payloads in order.
             ext_key = "{}/{}".format(ext, suffix)
-            json_key = dot + "json/{}".format(suffix)
-            dirname = bf.dirname
+            json_key = ".json/{}".format(suffix)
+            dirname = bf._dirname
 
             payloads = []
             ancestors = []
@@ -357,7 +351,8 @@ class BIDSLayoutIndexer:
                     if all(matches):
                         ancestors.append(path)
 
-                parent = os.path.dirname(dirname)
+                parent = dirname.parent
+
                 if parent == dirname:
                     break
                 dirname = parent
@@ -367,7 +362,7 @@ class BIDSLayoutIndexer:
 
             # Missing data files can tolerate absent metadata files,
             # but we will try to load it anyway
-            virtual_datafile = not os.path.exists(bf.path)
+            virtual_datafile = not bf._path.exists()
 
             # Create DB records for metadata associations
             js_file = payloads[0][1]
@@ -404,10 +399,10 @@ class BIDSLayoutIndexer:
             intended = listify(file_md.get('IntendedFor', []))
             for target in intended:
                 # Per spec, IntendedFor paths are relative to sub dir.
-                target = os.path.join(
-                    self._layout.root, 'sub-{}'.format(bf.entities['subject']),
+                target = self._layout._root.joinpath(
+                    'sub-{}'.format(bf.entities['subject']),
                     target)
-                create_association_pair(bf.path, target, 'IntendedFor',
+                create_association_pair(bf.path, str(target), 'IntendedFor',
                                         'InformedBy')
 
             # Link files to BOLD runs
@@ -445,9 +440,9 @@ class BIDSLayoutIndexer:
                             md_val))
                     continue
                 if md_key not in all_entities:
-                    all_entities[md_key] = Entity(md_key, is_metadata=True)
+                    all_entities[md_key] = Entity(md_key)
                     self.session.add(all_entities[md_key])
-                tag = Tag(bf, all_entities[md_key], md_val)
+                tag = Tag(bf, all_entities[md_key], md_val, is_metadata=True)
                 self.session.add(tag)
 
             if len(self.session.new) >= 1000:
