@@ -28,7 +28,6 @@ def test_layout_init(layout_7t_trt):
     assert isinstance(layout_7t_trt.files, dict)
 
 
-@pytest.mark.parametrize("extension_initial_dot", (True, False))
 @pytest.mark.parametrize(
     'index_metadata,query,result',
     [
@@ -49,6 +48,17 @@ def test_index_metadata(index_metadata, query, result, mock_config):
     assert metadata.get('RepetitionTime') == result
 
 
+
+@pytest.mark.parametrize('config_type', [str, Path])
+def test_config_filename(config_type):
+    data_path = Path(get_test_data_path())
+    # Use custom config that replaces session with oligarchy
+    config_path = data_path.parent / 'bids_specs_with_oligarchy.json'
+    layout = BIDSLayout(data_path / "7t_trt", config=config_type(config_path))
+    # Validate that we are using the desired configuration
+    assert 'oligarchy' in layout.get_entities()
+
+
 def test_layout_repr(layout_7t_trt):
     assert "Subjects: 10 | Sessions: 20 | Runs: 20" in str(layout_7t_trt)
 
@@ -501,7 +511,7 @@ def test_parse_file_entities_from_layout(layout_synthetic):
 
     # Test with entities taken from bids config
     target = {'subject': '03', 'session': '07', 'run': 4, 'suffix': 'sekret',
-              'extension': 'nii.gz'}
+              'extension': '.nii.gz'}
     assert target == layout.parse_file_entities(filename, config='bids')
     config = Config.load('bids')
     assert target == layout.parse_file_entities(filename, config=[config])
@@ -509,7 +519,7 @@ def test_parse_file_entities_from_layout(layout_synthetic):
 
     # Test with default scope--i.e., everything
     target = {'subject': '03', 'session': '07', 'run': 4, 'suffix': 'sekret',
-              'desc': 'bleargh', 'extension': 'nii.gz'}
+              'desc': 'bleargh', 'extension': '.nii.gz'}
     assert target == layout.parse_file_entities(filename)
     # Test with only the fmriprep pipeline (which includes both configs)
     assert target == layout.parse_file_entities(filename, scope='fmriprep')
@@ -702,6 +712,19 @@ def test_get_with_invalid_filters(layout_ds005):
     l.get(**filters)
 
 
+def test_get_with_query_constants_in_match_list(layout_ds005):
+    l = layout_ds005
+    get1 = l.get(subject='12', run=1, suffix='bold')
+    get_none = l.get(subject='12', run=None, suffix='bold')
+    get_any = l.get(subject='12', run=Query.ANY, suffix='bold')
+    get1_and_none = l.get(subject='12', run=[None, 1], suffix='bold')
+    get1_and_any = l.get(subject='12', run=[Query.ANY, 1], suffix='bold')
+    get_none_and_any = l.get(subject='12', run=[Query.ANY, Query.NONE], suffix='bold')
+    assert set(get1_and_none) == set(get1) | set(get_none)
+    assert set(get1_and_any) == set(get1) | set(get_any)
+    assert set(get_none_and_any) == set(get_none) | set(get_any)
+
+
 def test_load_layout(layout_synthetic_nodb, db_dir):
     db_path = str(db_dir / 'tmp_db')
     layout_synthetic_nodb.save(db_path)
