@@ -1,6 +1,6 @@
 """Functionality related to validation of BIDSLayouts and BIDS projects."""
 
-import os
+from pathlib import Path
 import json
 import re
 import warnings
@@ -17,8 +17,8 @@ MANDATORY_BIDS_FIELDS = {
 
 MANDATORY_DERIVATIVES_FIELDS = {
     **MANDATORY_BIDS_FIELDS,
-    "PipelineDescription.Name": {
-        "PipelineDescription": {"Name": "Example pipeline"}
+    "GeneratedBy": {
+        "GeneratedBy": [{"Name": "Example pipeline"}]
     },
 }
 
@@ -55,20 +55,20 @@ def indexer_arg_deprecation_warning():
 def validate_root(root, validate):
     # Validate root argument and make sure it contains mandatory info
     try:
-        root = str(root)
-    except:
-        raise TypeError("root argument must be a string (or a type that "
-                        "supports casting to string, such as "
-                        "pathlib.Path) specifying the directory "
+        root = Path(root)
+    except TypeError:
+        raise TypeError("root argument must be a pathlib.Path (or a type that "
+                        "supports casting to pathlib.Path, such as "
+                        "string) specifying the directory "
                         "containing the BIDS dataset.")
 
-    root = os.path.abspath(root)
+    root = root.absolute()
 
-    if not os.path.exists(root):
+    if not root.exists():
         raise ValueError("BIDS root does not exist: %s" % root)
 
-    target = os.path.join(root, 'dataset_description.json')
-    if not os.path.exists(target):
+    target = root / 'dataset_description.json'
+    if not target.exists():
         if validate:
             raise BIDSValidationError(
                 "'dataset_description.json' is missing from project root."
@@ -79,11 +79,13 @@ def validate_root(root, validate):
         else:
             description = None
     else:
+        err = None
         try:
             with open(target, 'r', encoding='utf-8') as desc_fd:
                 description = json.load(desc_fd)
-        except json.JSONDecodeError:
+        except (UnicodeDecodeError, json.JSONDecodeError) as e:
             description = None
+            err = e
         if validate:
 
             if description is None:
@@ -92,8 +94,8 @@ def validate_root(root, validate):
                     " There is likely a typo in your 'dataset_description.json'."
                     "\nExample contents of 'dataset_description.json': \n%s" %
                     json.dumps(EXAMPLE_BIDS_DESCRIPTION)
-                )
-            
+                ) from err
+
             for k in MANDATORY_BIDS_FIELDS:
                 if k not in description:
                     raise BIDSValidationError(
@@ -111,25 +113,21 @@ def validate_derivative_paths(paths, layout=None, **kwargs):
 
     # Collect all paths that contain a dataset_description.json
     def check_for_description(bids_dir):
-        dd = os.path.join(bids_dir, 'dataset_description.json')
-        return os.path.exists(dd)
+        dd = bids_dir / 'dataset_description.json'
+        return dd.exists()
 
     for p in paths:
-        p = os.path.abspath(str(p))
-        if os.path.exists(p):
+        p = Path(p).absolute()
+        if p.exists():
             if check_for_description(p):
                 deriv_dirs.append(p)
             else:
-                subdirs = [d for d in os.listdir(p)
-                            if os.path.isdir(os.path.join(p, d))]
-                for sd in subdirs:
-                    sd = os.path.join(p, sd)
-                    if check_for_description(sd):
-                        deriv_dirs.append(sd)
+                deriv_dirs.extend(sd for sd in p.iterdir()
+                                  if sd.is_dir() and check_for_description(sd))
 
     if not deriv_dirs:
         warnings.warn("Derivative indexing was requested, but no valid "
-                        "datasets were found in the specified locations "
+                        "derivative datasets were found in the specified locations "
                         "({}). Note that all BIDS-Derivatives datasets must"
                         " meet all the requirements for BIDS-Raw datasets "
                         "(a common problem is to fail to include a "
@@ -141,18 +139,27 @@ def validate_derivative_paths(paths, layout=None, **kwargs):
     paths = {}
 
     for deriv in deriv_dirs:
-        dd = os.path.join(deriv, 'dataset_description.json')
-        with open(dd, 'r', encoding='utf-8') as ddfd:
+        dd = deriv / 'dataset_description.json'
+        with dd.open('r', encoding='utf-8') as ddfd:
             description = json.load(ddfd)
-        pipeline_name = description.get(
-            'PipelineDescription', {}).get('Name')
+        pipeline_names = [pipeline["Name"]
+                          for pipeline in description.get("GeneratedBy", [])
+                          if "Name" in pipeline]
+        if pipeline_names:
+            pipeline_name = pipeline_names[0]
+        elif "PipelineDescription" in description:
+            warnings.warn("The PipelineDescription field was superseded "
+                          "by GeneratedBy in BIDS 1.4.0. You can use "
+                          "``pybids upgrade`` to update your derivative "
+                          "dataset.")
+            pipeline_name = description["PipelineDescription"].get("Name")
         if pipeline_name is None:
             raise BIDSDerivativesValidationError(
                                 "Every valid BIDS-derivatives dataset must "
-                                "have a PipelineDescription.Name field set "
+                                "have a GeneratedBy.Name field set "
                                 "inside 'dataset_description.json'. "
                                 "\nExample: %s" %
-                                MANDATORY_DERIVATIVES_FIELDS['PipelineDescription.Name'])
+                                MANDATORY_DERIVATIVES_FIELDS['GeneratedBy'])
         if layout is not None and pipeline_name in layout.derivatives:
             raise BIDSDerivativesValidationError(
                                 "Pipeline name '%s' has already been added "
@@ -167,11 +174,11 @@ def validate_indexing_args(ignore, force_index, root):
     if ignore is None:
         ignore = DEFAULT_LOCATIONS_TO_IGNORE
 
-    # Do after root validation to ensure os.path.join works
-    ignore = [os.path.abspath(os.path.join(root, patt))
-                    if isinstance(patt, str) else patt
-                    for patt in listify(ignore or [])]
-    force_index = [os.path.abspath(os.path.join(root, patt))
+    # root has already been validated to be a directory
+    ignore = [(root / patt).absolute()
+              if isinstance(patt, str) else patt
+              for patt in listify(ignore or [])]
+    force_index = [(root / patt).absolute()
                    if isinstance(patt, str) else patt
                    for patt in listify(force_index or [])]
 
@@ -179,7 +186,7 @@ def validate_indexing_args(ignore, force_index, root):
     if force_index is not None:
         for entry in force_index:
             condi = (isinstance(entry, str) and
-                        os.path.normpath(entry).startswith('derivatives'))
+                     str(entry.resolve()).startswith('derivatives'))
             if condi:
                 msg = ("Do not pass 'derivatives' in the force_index "
                         "list. To index derivatives, either set "
