@@ -11,7 +11,6 @@ import numpy as np
 import pandas as pd
 
 from bids.utils import listify
-from bids.utils import matches_entities
 
 class BIDSVariable(metaclass=ABCMeta):
     """Base representation of a column in a BIDS project. """
@@ -496,7 +495,7 @@ class DenseRunVariable(BIDSVariable):
                                  source=self.source,
                                  sampling_rate=self.sampling_rate)
                 for i, name in enumerate(df.columns)]
-        
+
     def _build_entity_index(self, run_info, sampling_rate, match_vol=False):
         """Build the entity index from run information. """
 
@@ -509,7 +508,7 @@ class DenseRunVariable(BIDSVariable):
             else:
                 reps = int(math.ceil(run.duration * sampling_rate))
 
-            interval = int(round(1000. / sampling_rate))            
+            interval = int(round(1000. / sampling_rate))
             ent_vals = list(run.entities.values())
             df = pd.DataFrame([ent_vals] * reps, columns=list(run.entities.keys()))
             ts = pd.date_range(0, periods=len(df), freq='%sms' % interval)
@@ -537,19 +536,19 @@ class DenseRunVariable(BIDSVariable):
             var = self.clone()
             var.resample(sampling_rate, True, kind)
             return var
-        
+
         match_vol = False
         if sampling_rate == 'TR':
             match_vol = True
             sampling_rate = 1. / self.run_info[0].tr
-        
+
         if sampling_rate == self.sampling_rate:
             return
-        
+
         n = len(self.index)
 
         self.index = self._build_entity_index(self.run_info, sampling_rate, match_vol)
-        
+
         x = np.arange(n)
         num = len(self.index)
 
@@ -627,16 +626,13 @@ class DenseRunVariable(BIDSVariable):
             sampling_rate=sampling_rate)
 
 
-def merge_variables(variables, name=None, **kwargs):
+def merge_variables(variables, **kwargs):
     """Merge/concatenate a list of variables along the row axis.
 
     Parameters
     ----------
     variables : :obj:`list`
         A list of Variables to merge.
-    name : :obj:`str`
-        Optional name to assign to the output Variable. By default, uses the
-        same name as the input variables.
     kwargs
         Optional keyword arguments to pass onto the class-specific merge() call.
         Possible args:
